{% load i18n %}

<div class="detail">
  <dl class="dl-horizontal">

    <dt>{% trans "ID" %}</dt>
    <dd>{{ runtime.id }}</dd>

    <dt>{% trans "Name" %}</dt>
    <dd data-display="{{ runtime.name }}">{{ runtime.name }}</dd>

    <dt>{% trans "Image" %}</dt>
    <dd>{{ runtime.image }}</dd>

    <dt>{% trans "Description" %}</dt>
    <dd>{{ runtime.description }}</dd>

    <dt>{% trans "Project ID" %}</dt>
    <dd>{{ runtime.project_id }}</dd>

    <dt>{% trans "Created At" %}</dt>
    <dd>{{ runtime.created_at|parse_isotime }}</dd>

    <dt>{% trans "Updated At" %}</dt>
    <dd>{{ runtime.updated_at|parse_isotime }}</dd>

    {% if is_superuser %}
      <dt>{% trans "Is Public" %}</dt>
      <dd>{{ runtime.is_public|default:_("Unknown") }}</dd>
    {% endif %}

    <dt>{% trans "Status" %}</dt>
    {% with runtime.status|title as rt_status %}
    <dd>{% trans rt_status context "current status of a runtime" %}</dd>
    {% endwith %}

  </dl>
</div>
