{% load i18n %}

<div class="detail">
  <dl class="dl-horizontal">

    <dt>{% trans "ID" %}</dt>
    <dd>{{ version.id }}</dd>

    <dt>{% trans "Function ID" %}</dt>
    <dd>{{ version.function_id }}</dd>

    <dt>{% trans "Description" %}</dt>
    <dd>{{ version.description }}</dd>

    <dt>{% trans "Version Number" %}</dt>
    <dd>{{ version.version_number }}</dd>

    <dt>{% trans "Count" %}</dt>
    <dd>{{ version.count }}</dd>

    <dt>{% trans "Project ID" %}</dt>
    <dd>{{ version.project_id }}</dd>

    <dt>{% trans "Created At" %}</dt>
    <dd>{{ version.created_at|parse_isotime }}</dd>

    <dt>{% trans "Updated At" %}</dt>
    <dd>{{ version.updated_at|parse_isotime }}</dd>

  </dl>
</div>
